@@ -1,5 +1,6 @@
 import cPickle
 import logging
+import pickle
 from datetime import datetime
 
 from beaker.container import OpenResourceNamespaceManager, Container
@@ -7,30 +8,34 @@ from beaker.exceptions import InvalidCacheBackendError, MissingCacheParameter
 from beaker.synchronization import file_synchronizer, null_synchronizer
 from beaker.util import verify_directory, SyncDict
 
-sa_version = None
-
 log = logging.getLogger(__name__)
 
-try:
-    import sqlalchemy as sa
-    import sqlalchemy.pool as pool
-    from sqlalchemy import types
-    sa_version = '0.3'
-except ImportError:
-    raise InvalidCacheBackendError("Database cache backend requires the 'sqlalchemy' library")
-
-if not hasattr(sa, 'BoundMetaData'):
-    sa_version = '0.4'
+sa = None
+pool = None
+types = None
 
 class DatabaseNamespaceManager(OpenResourceNamespaceManager):
     metadatas = SyncDict()
     tables = SyncDict()
-    
+
+    @classmethod
+    def _init_dependencies(cls):
+        global sa, pool, types
+        if sa is not None:
+            return
+        try:
+            import sqlalchemy as sa
+            import sqlalchemy.pool as pool
+            from sqlalchemy import types
+        except ImportError:
+            raise InvalidCacheBackendError("Database cache backend requires "
+                                            "the 'sqlalchemy' library")
+
     def __init__(self, namespace, url=None, sa_opts=None, optimistic=False,
                  table_name='beaker_cache', data_dir=None, lock_dir=None,
                  **params):
         """Creates a database namespace manager
-        
+
         ``url``
             SQLAlchemy compliant db url
         ``sa_opts``
@@ -44,7 +49,7 @@ class DatabaseNamespaceManager(OpenResourceNamespaceManager):
             The table name to use in the database for the cache.
         """
         OpenResourceNamespaceManager.__init__(self, namespace)
-        
+
         if sa_opts is None:
             sa_opts = params
 
@@ -53,8 +58,8 @@ class DatabaseNamespaceManager(OpenResourceNamespaceManager):
         elif data_dir:
             self.lock_dir = data_dir + "/container_db_lock"
         if self.lock_dir:
-            verify_directory(self.lock_dir)            
-        
+            verify_directory(self.lock_dir)
+
         # Check to see if the table's been created before
         url = url or sa_opts['sa.url']
         table_key = url + table_name
@@ -62,18 +67,12 @@ class DatabaseNamespaceManager(OpenResourceNamespaceManager):
             # Check to see if we have a connection pool open already
             meta_key = url + table_name
             def make_meta():
-                if sa_version == '0.3':
-                    if url.startswith('mysql') and not sa_opts:
-                        sa_opts['poolclass'] = pool.QueuePool
-                    engine = sa.create_engine(url, **sa_opts)
-                    meta = sa.BoundMetaData(engine)
-                else:
-                    # SQLAlchemy pops the url, this ensures it sticks around
-                    # later
-                    sa_opts['sa.url'] = url
-                    engine = sa.engine_from_config(sa_opts, 'sa.')
-                    meta = sa.MetaData()
-                    meta.bind = engine
+                # SQLAlchemy pops the url, this ensures it sticks around
+                # later
+                sa_opts['sa.url'] = url
+                engine = sa.engine_from_config(sa_opts, 'sa.')
+                meta = sa.MetaData()
+                meta.bind = engine
                 return meta
             meta = DatabaseNamespaceManager.metadatas.get(meta_key, make_meta)
             # Create the table object and cache it now
@@ -82,7 +81,7 @@ class DatabaseNamespaceManager(OpenResourceNamespaceManager):
                              sa.Column('namespace', types.String(255), nullable=False),
                              sa.Column('accessed', types.DateTime, nullable=False),
                              sa.Column('created', types.DateTime, nullable=False),
-                             sa.Column('data', types.BLOB(), nullable=False),
+                             sa.Column('data', types.PickleType, nullable=False),
                              sa.UniqueConstraint('namespace')
             )
             cache.create(checkfirst=True)
@@ -91,7 +90,7 @@ class DatabaseNamespaceManager(OpenResourceNamespaceManager):
         self._is_new = False
         self.loaded = False
         self.cache = DatabaseNamespaceManager.tables.get(table_key, make_cache)
-    
+
     def get_access_lock(self):
         return null_synchronizer()
 
@@ -100,12 +99,12 @@ class DatabaseNamespaceManager(OpenResourceNamespaceManager):
             identifier ="databasecontainer/funclock/%s" % self.namespace,
             lock_dir = self.lock_dir)
 
-    def do_open(self, flags):
+    def do_open(self, flags, replace):
         # If we already loaded the data, don't bother loading it again
         if self.loaded:
             self.flags = flags
             return
-        
+
         cache = self.cache
         result = sa.select([cache.c.data], 
                            cache.c.namespace==self.namespace
@@ -116,33 +115,33 @@ class DatabaseNamespaceManager(OpenResourceNamespaceManager):
         else:
             self._is_new = False
             try:
-                self.hash = cPickle.loads(str(result['data']))
-            except (IOError, OSError, EOFError, cPickle.PickleError):
+                self.hash = result['data']
+            except (IOError, OSError, EOFError, cPickle.PickleError,
+                    pickle.PickleError):
                 log.debug("Couln't load pickle data, creating new storage")
                 self.hash = {}
                 self._is_new = True
         self.flags = flags
         self.loaded = True
-    
+
     def do_close(self):
         if self.flags is not None and (self.flags == 'c' or self.flags == 'w'):
             cache = self.cache
             if self._is_new:
-                cache.insert().execute(namespace=self.namespace, 
-                                       data=cPickle.dumps(self.hash),
-                                       accessed=datetime.now(), 
+                cache.insert().execute(namespace=self.namespace, data=self.hash,
+                                       accessed=datetime.now(),
                                        created=datetime.now())
                 self._is_new = False
             else:
                 cache.update(cache.c.namespace==self.namespace).execute(
-                    data=cPickle.dumps(self.hash), accessed=datetime.now())
+                    data=self.hash, accessed=datetime.now())
         self.flags = None
-    
+
     def do_remove(self):
         cache = self.cache
         cache.delete(cache.c.namespace==self.namespace).execute()
         self.hash = {}
-        
+
         # We can retain the fact that we did a load attempt, but since the
         # file is gone this will be a new namespace should it be saved.
         self._is_new = True
@@ -152,7 +151,7 @@ class DatabaseNamespaceManager(OpenResourceNamespaceManager):
 
     def __contains__(self, key): 
         return self.hash.has_key(key)
-        
+
     def __setitem__(self, key, value):
         self.hash[key] = value
 
@@ -163,4 +162,4 @@ class DatabaseNamespaceManager(OpenResourceNamespaceManager):
         return self.hash.keys()
 
 class DatabaseContainer(Container):
-    namespace_manager = DatabaseNamespaceManager
\ No newline at end of file
+    namespace_manager = DatabaseNamespaceManager
